capture program drop change
*! version 1.2.0 6/16/97 - use catnm no setcat
*! version 1.1.9 6/10/97 - list std dev of x and check setcat
*! version 1.1.9 6/12/97 - check setcat
*! version 1.1.8 4/9/97 - use getrhs
* version 1.1.7 4/9/97 - add nbreg
* version 1.1.5 4/9/97 - add poisson
* version 1.1.4 4/7/97 - changes to work with new p_val
* version 1.1.3 10/28/96
*
* -change- computes a table of discrete change and marginal
*  change for logit, probit, ologit, oprobit, mlogit, poisson,
*  nbreg
*
* Results are saved to matrix jsl_chng:
*
*   if binary, rows are variables,
*              cols are min>max, 0>1, -+1/2, -+sd/2, marg effect
*
*   if ord or nominal,
*              cols are categories
*              rows are in sets of 5 per variable containing:
*                min>max, 0>1, -+1/2, -+sd/2, marg effect
*
* do: should jsl_base leave whatever is used as base values?
* do: if and in?
*

program define change
    version 5.0

    if ("$S_E_cmd"!="poisson" /*
    */ & "$S_E_cmd"!="logit" & "$S_E_cmd"!="probit" /*
    */ & "$S_E_cmd"!="ologit" & "$S_E_cmd"!="oprobit" /*
    */ & "$S_E_cmd"!="nbreg" & "$S_E_cmd"!="mlogit" /*
    */ ) {
di in r "-change- only works for logit, mlogit, nbreg, ologit, oprobit, & poisson."
     exit } 
    
    tempvar out base basepr expval lo hi dif xsd xmin xmax marg b
    parse "`*'", parse(" ")
    local opt="`1'"

* get number of variables and b's for most models
    mat `b' = get(_b)
    if "$S_E_cmd"=="nbreg" {       /* strip off alpha */
        local nb = colsof(`b') - 1 /* -1 for alpha */
        mat `b' = `b'[1,1..`nb']
    }
    local nvars = colsof(`b') /* # of rhs & constants */

* determine # of lhs categories if ordered regression model
    if "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" {
        local ncats 1
        capture local j=_b[_cut`ncats']
        while _rc==0 {
            local ncats=`ncats'+1
            capture local j=_b[_cut`ncats']
        }
    local nvars = `nvars' - `ncats' + 2 /* don't count cutpoints */
    }
    if "$S_E_cmd"=="mlogit" {
        local ncats = rowsof(`b') + 1
    }

* get names
    getrhs
    local rhsnam "$S_1"

* get desc stats for rhs variables
    quietly getdesc
    mat `xsd' = S_2[1,2...]
    mat `xmin' = S_3[1,2...]
    mat `xmax' = S_4[1,2...]

* create jsl_base
    mat `base' = S_2 /* add sd to be listed below */
    if "`opt'"=="" { /* use mean by default */
        mat `base' = S_1 \ `base'
        mat rownames `base' = X=mean StdDev
    }
    else if "`opt'"=="base" {
        capture local nbase = colsof(jsl_base)
        if "`nbase'"~="`nvars'" {
            di in red "# of base values is wrong."
            tobase
            exit
        }
        mat `base' = jsl_base \ `base' /* add to SDs */
        mat rownames `base' = X=base StdDev
    }
    else {
        di in red "Invalid option for -change-."
        exit
    }
    mat `base' = `base'[1..2,2...] /* get rid of lhs */
    local nvars = `nvars' - 1 /* remove lhs var */

* get expectations at base level
    mat S_1 = `base'[1,1...]
    if "$S_E_cmd"=="nbreg" | "$S_E_cmd"=="poisson" {
        e_val
    }
    else {
        p_val
    }
    mat `basepr' = S_2
    mat `expval' = S_1
    mat rownames `basepr' = BaseProb

* get marginals
    mat S_1 = `base'[1,1...]
    m_val
    mat `marg' = S_1
    scalar `lo' = 1/2
    mat `xsd'= `lo'*`xsd' /* xsd has sd/2 */

* header information
    di ""
    if "$S_E_cmd"=="poisson" | "$S_E_cmd"=="nbreg" {
        di in g "Changes in Predicted Rate"
    }
    else {
        di in g "Changes in Predicted Probabilities"
    }
    di
    local l 0
    if "$S_E_cmd"=="logit" | "$S_E_cmd"=="probit" { local l 5 }
    di in g "Procedure:    " _skip(`l') in y "$S_E_cmd"
    di in g "LHS variable: " _skip(`l') in y "$S_E_depv"

* loop over variables for count models to compute Pr(y=i)
    if "$S_E_cmd"=="poisson" | "$S_E_cmd"=="nbreg" {
        mat `out' = J(`nvars',4,1)
        local i = 1
        while `i'<=`nvars' {
            local i1 = `i' + 1
        * min->max
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = `xmin'[1,`i']
            e_val
            scalar `lo' = S_1[1,1]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = `xmax'[1,`i']
            e_val
            scalar `dif' = S_1[1,1] - `lo'
            mat `out'[`i',1] = `dif'
        * 0-->1
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = 0
            e_val
            scalar `lo' = S_1[1,1]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = 1
            e_val
            scalar `dif' = S_1[1,1] - `lo'
            mat `out'[`i',2] = `dif'
        * -/+.5
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] - .5
            e_val
            scalar `lo' = S_1[1,1]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] + .5
            e_val
            scalar `dif' = S_1[1,1] - `lo'
            mat `out'[`i',3] = `dif'
        * -/+.5sd
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] - `xsd'[1,`i']
            e_val
            scalar `lo' = S_1[1,1]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] + `xsd'[1,`i']
            e_val
            scalar `dif' = S_1[1,1] - `lo'
            mat `out'[`i',4] = `dif'
            local i=`i'+1
        }
        mat `marg' = `marg'[2,.]  /* get marg wrt to a 1 */
        mat `marg' = `marg''
        mat `out' = `out' , `marg'
        mat colnames `out' = Min->Max 0->1 -+1/2 -+sd/2 MargEfct
        mat rownames `out' = `rhsnam'
        mat list `out',noheader f(%8.4f)
        local expv `expval'[1,1]
        di
        di in g "exp(xb):" in y %8.4f `expv'
        mat list `base',noheader f(%8.4f)
        mat jsl_chng = `out'
    }

* loop over variables for binary models
    if "$S_E_cmd"=="logit" | "$S_E_cmd"=="probit" {
        quietly catnm $S_E_depv
        local catnm "$S_1"
        mat `out' = J(`nvars',4,1)
        mat colnames `basepr' = `catnm'
        local nm : word 2 of `catnm'
        di ""
        di in g "Change in outcome: " in y "`nm'"
        local i = 1
        while `i'<=`nvars' {
            local i1 = `i' + 1
        * min->max
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = `xmin'[1,`i']
            p_val
            scalar `lo' = S_2[1,2]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = `xmax'[1,`i']
            p_val
            scalar `dif' = S_2[1,2] - `lo'
            mat `out'[`i',1] = `dif'
        * 0-->1
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = 0
            p_val
            scalar `lo' = S_2[1,2]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = 1
            p_val
            scalar `dif' = S_2[1,2] - `lo'
            mat `out'[`i',2] = `dif'
        * -/+.5
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] - .5
            p_val
            scalar `lo' = S_2[1,2]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] + .5
            p_val
            scalar `dif' = S_2[1,2] - `lo'
            mat `out'[`i',3] = `dif'
        * -/+.5sd
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] - `xsd'[1,`i']
            p_val
            scalar `lo' = S_2[1,2]
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] + `xsd'[1,`i']
            p_val
            scalar `dif' = S_2[1,2] - `lo'
            mat `out'[`i',4] = `dif'
            local i=`i'+1
        }
        mat `marg' = `marg'[2,.]  /* get marg wrt to a 1 */
        mat `marg' = `marg''
        mat `out' = `out' , `marg'
        mat colnames `out' = Min->Max 0->1 -+1/2 -+sd/2 MargEfct
        mat rownames `out' = `rhsnam'
        mat list `out',noheader f(%8.4f)
        mat list `basepr', noheader f(%8.4f)
        mat list `base',noheader f(%8.4f)
        mat jsl_chng = `out'
    }

* loop over variables if ologit, oprobit and mlogit
    if "$S_E_cmd"=="ologit" | "$S_E_cmd"=="oprobit" /*
        */ | "$S_E_cmd"=="mlogit" {
        mat `out' = J(5,`ncats',1)
        mat rownames `out' = Min->Max 0->1 -+1/2 -+sd/2 MargEfct
        parse "`rhsnam'", parse (" ")
        quietly catnm $S_E_depv
        local catnm "$S_1"
        mat colnames `basepr' = `catnm'
        local i = 1
        while `i'<=`nvars' {
            di ""
            di in white "`1'",_continue
            macro shift
            local i1 = `i' + 1
        * min->max
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = `xmin'[1,`i']
            p_val
            mat `lo' = S_2
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = `xmax'[1,`i']
            p_val
            mat `dif' = S_2 - `lo'
            local j = 1
            while `j' <= `ncats' {
                mat `out'[1,`j'] = `dif'[1,`j']
                local j = `j' + 1 }
        * 0-->1
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = 0
            p_val
            mat `lo' = S_2
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = 1
            p_val
            mat `dif' = S_2 - `lo'
            local j = 1
            while `j' <= `ncats' {
                mat `out'[2,`j'] = `dif'[1,`j']
                local j = `j' + 1 }
        * -/+.5
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] - .5
            p_val
            mat `lo' = S_2
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] + .5
            p_val
            mat `dif' = S_2 - `lo'
            local j = 1
            while `j' <= `ncats' {
                mat `out'[3,`j'] = `dif'[1,`j']
                local j = `j' + 1 }
        * -/+.5sd
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] - `xsd'[1,`i']
            p_val
            mat `lo' = S_2
            mat S_1 = `base'[1,1...]
            mat S_1[1,`i'] = S_1[1,`i'] + `xsd'[1,`i']
            p_val
            mat `dif' = S_2 - `lo'
            local j = 1
            while `j' <= `ncats' {
                mat `out'[4,`j'] = `dif'[1,`j']
                local j = `j' + 1 }
        * marginal
            local j = 1
            while `j' <= `ncats' {
                mat `out'[5,`j'] = `marg'[`i',`j']
                local j = `j' + 1 }
                * write it
                mat colnames `out' = `catnm'
                mat list `out', noheader f(%8.4f)
                if `i' == 1 {
                    mat jsl_chng = `out'
                    }
                else if `i' != 1 {
                    mat jsl_chng = jsl_chng \ `out'
                }
            local i=`i'+1
        }
        mat list `basepr',f(%8.4f) noheader
        mat list `base',noheader f(%8.4f)
    }
end

