capture program drop chngplot
*! version 1.0.0 6/16/97 - use catnm
*! version 0.0.2 4/12/97
*
* chngplot - discrete change plot of -mlogit-
*
* chngplot ([# r|0|1|s],[# ]...)
*
* Note: you can list the same variable more than once.
* Note: maximum of 11 sets of change coefficients in one plot

program define chngplot
    version 5.0
    local input = "`*'"

* this allow running the program several times after a mlogit
    capture drop cplt cvar ccat ctype cltr cchange rchange
    tempvar b chrng ch01 ch1 chsd tmp istype toplot

* get data from logit
    if "$S_E_cmd"!="mlogit" {
        di in red "-chngplot- must be run after -mlogit-."
        exit
    }

* check that jsl_chng is there
    capture quietly mat list jsl_chng
    if _rc != 0 {
        di in red "-chngplot- must be run after -change- has been run."
        exit
    }

* get coefficient data and names.
    mat `b' = get(_b)
    local varnm : colnames(`b')
    parse "`varnm'", parse (" ")
    local nvarp1 = colsof(`b') /* with constant */
    local nvar = `nvarp1' - 1 /* excludes constant */
    local ncat = rowsof(`b') + 1
    local maxnvar = 4*`nvar' /* can plot each var 4 ways */

* grab change coefficients
    local ivar = 1
    mat `chrng' = J(`nvar',`ncat',0)
    mat `ch01' = J(`nvar',`ncat',0)
    mat `ch1' = J(`nvar',`ncat',0)
    mat `chsd' = J(`nvar',`ncat',0)
    local r = 1
    while `ivar' <= `nvar' {
        local c = 1
        while `c' <= `ncat' {
            scalar `tmp' = jsl_chng[`r',`c']
            mat `chrng'[`ivar',`c'] = `tmp'
            scalar `tmp' = jsl_chng[`r'+1,`c']
            mat `ch01'[`ivar',`c'] = `tmp'
            scalar `tmp' = jsl_chng[`r'+2,`c']
            mat `ch1'[`ivar',`c'] = `tmp'
            scalar `tmp' = jsl_chng[`r'+3,`c']
            mat `chsd'[`ivar',`c'] = `tmp'
            local c = `c' + 1
        }
        local ivar = `ivar' + 1
        local r = `r' + 5
    }

* parse type of change information to plot
    mat `istype' = J(`maxnvar',1,0)  /* 1=rng, 2=01, 3=+-1, 4=+-sd */
    mat `toplot' = J(`maxnvar',1,0)  /* 1 if plot this coef */
    local nplot = 0
    parse "`input'", parse(" ")
    local maxvar = 11
    * if not variables specifed, take first maxvar and plot ch1
    if "`1'"=="" {
        local ivar = 1
        while `ivar' <= `nvar' {
            if `ivar' <= `maxvar' {
                mat `toplot'[`ivar',1] = `ivar'
                mat `istype'[`ivar',1] = 3 /* type +-1 by default */
                }
            local ivar = `ivar' + 1
            }
        local nplot = `ivar' - 1
        }
    else {
        while "`1'"~="" {
           if `nplot' >= `maxvar' {
di in red "Only `maxvar' variables can be plotted at one time."
            exit
            }
            if `1' > `nvar' {
                di in red "Specified variable number is too large."
                exit
                }
            else {
                local ntoplot = `ntoplot' + 1
                mat `toplot'[`ntoplot',1] = `1' /* plt var in this order */
            }
            macro shift
            * decide which type of coefficient to plot            
            if "`1'" == "r" {
                mat `istype'[`ntoplot',1] = 1
            }
            else if "`1'" == "0" {
                mat `istype'[`ntoplot',1] = 2
            }
            else if "`1'" == "1" {
                mat `istype'[`ntoplot',1] = 3
            }
            else if "`1'" == "s" {
                mat `istype'[`ntoplot',1] = 4
            }
            macro shift
            local nplot = `nplot' + 1
            }
        }

* stack coefficients to plot
    tempvar cplt cvar ccat ctype
    local nout = `nplot' * `ncat'  /* # coefs to plot */
    matrix `cplt' = J(`nout',1,0)  /* coef to plot */
    matrix `cvar' = J(`nout',1,0)  /* variable nubmer */
    matrix `ccat' = J(`nout',1,0)  /* category */
    matrix `ctype' = J(`nout',1,0)  /* 1=rng, 2=01, 3=+-1 4=+-sd */

* loop through variables, decide on type of coef to plot
    local iloc = 1
    local ivar = 1
    while `ivar' <= `nplot' {
        local icat = 1
        local varnum = `toplot'[`ivar',1]
        while `icat' <= `ncat' {
            if `istype'[`ivar',1] == 1 { /* range */
                matrix `cplt'[`iloc',1] = `chrng'[`varnum',`icat']
                matrix `ctype'[`iloc',1] = 1                
            }
            else if `istype'[`ivar',1] == 2 { /* 0 to 1 */
                matrix `cplt'[`iloc',1] = `ch01'[`varnum',`icat']
                matrix `ctype'[`iloc',1] = 2                                
            }
            else if `istype'[`ivar',1] == 3 { /* +-1 */
                matrix `cplt'[`iloc',1] = `ch1'[`varnum',`icat']
                matrix `ctype'[`iloc',1] = 3                                
            }
            else if `istype'[`ivar',1] == 4 { /* +-sd */
                matrix `cplt'[`iloc',1] = `chsd'[`varnum',`icat']
                matrix `ctype'[`iloc',1] = 4                                
            }
            * cplt now has coefficients to be plotted
            * ctype now the type of coefficient it is
            matrix `cvar'[`iloc',1] = `ivar' /* var# for given coef */
            matrix `ccat'[`iloc',1] = `icat' /* cat# for tiven coef */
            local iloc = `iloc' + 1
            local icat = `icat' + 1
        }
        local ivar = `ivar' + 1
    }

* create variables from the matrices
    mat colnames `cplt' = cplt
    mat colnames `cvar' = cvar
    mat colnames `ccat' = ccat
    mat colnames `ctype' = ctype
    svmat `cplt', names(col)
    svmat `cvar', names(col)
    svmat `ccat', names(col)
    svmat `ctype', names(col)

* generate variable with first letter of category names

    quietly catnm $S_E_depv
    local refcat "$S_2"
    local catnm  "$S_1"
    local ltr : word `refcat' of `catnm'                
    local ltr = upper(substr("`ltr'",1,1))
    quietly generate str1 cltr = "`ltr'"

*    local ltr = upper(substr("$jsl_rfnm",1,1))
*    quietly generate str1 cltr = "`ltr'"

    local icat = 1
    while `icat' <= `ncat' {
        if `icat'!=`refcat' {    
            local nm: word `icat' of `catnm'
            local ltr = upper(substr("`nm'",1,1))
            quietly replace cltr = "`ltr'" if ccat == `icat'
        }
        local icat = `icat' + 1        
    }
*
* plot results in this coordinate space:
*
*   (0,0)       (0,cstrt)                   (0,cend)
*
*               (rhead,cstrt)      data     (rhead,cend)
*
*               (rhead+rvar,cstrt)          (rhead+rvar,cend)
*
* parameters used for locating plot
*
    local cstrt  = 10000  /* first column in plot space */
    local cborder = 1000  /* border at ends of plot space */
    local cend   = 31500  /* right most column of plot space */
    local crange = `cend' - `cstrt' - (2*`cborder')
    local cname  = `cstrt' - 700  /* name of variable ends here */
    local rhead  = 1000   /* header space at top of graph - bigger moves it down */
    local rvar   = 1800   /* rows per variable in plot space */    
    local rnmoff = 1000 - `rvar'  /* vert offset for listing name */
    local rltoff = 1600   /* vert offset top & bot for letters */    
    local ntic = 7        /* number tic marks */

* rescale to metric used in columns of plot
    quietly sum cplt /* get min and max of coefficients being plotted */
    local minb = _result(5)
    local maxb = _result(6)
    local rng = `maxb' - `minb'
    * change range to 0--1
    quietly generate cchange = (cplt - `minb')/`rng'
    local at0 = -`minb'/`rng'
* min and max column for plotting area
    local minn = `cstrt' + `cborder'
    local maxx = `cend' - `cborder'
    local rng = `maxx' - `minn'
    quietly replace cchange = (cchange * `rng') + `minn'
    local at0 = (`at0'*`rng') + `minn'
    local rltr = `rvar' - `rltoff' - `rltoff'
    #delimit ;
    generate rchange = 
                  + `rhead'            /* skip over header */
                  + ((cvar-1)*`rvar')  /* space over prior variables */
                  + `rltoff'           /* space before letters begin */
                  - 650; /* -bigger moves all letters up from base line */
    #delimit cr

* make the graph
    gph open
    gph font 600 300

* axis at top of graph
    local rloc = `rhead' - 200
    gph line `rloc' `cstrt' `rloc' `cend'
/*
* label at top of graph
    gph font 500 250
    local rloc = `rhead' - 1400
*/

* label at bottom of graph
    local ltr = upper("$jsl_rfnm")
    local rloc = `rhead' + ((`ivar'-1)*`rvar') + 1400
    gph text `rloc' `cstrt' 0 -1 Change in the Predicted Probability

* add tic marks
    local nper = (`maxb' - `minb')/(`ntic' - 1)
    local itic = 1
    local minn = `cstrt' + `cborder'
    local maxx = `cend' - `cborder'
    local cper = (`maxx' - `minn')/(`ntic' - 1)

* vertical line at 0    
    local rtop = `rhead' - 200
    local rlend = `rhead' + (`nplot'*`rvar') - 200
    gph line `rtop' `at0' `rlend' `at0'    
    while `itic' <= `ntic' {
        local cval = `minn' + ((`itic'-1)*`cper')
        local rloc = `rhead' - 600
/*
* upper tic numbers        
        local ticval : display round(exp(`minb' + ((`itic'-1)*`nper')),.01)        
        gph text `rloc' `cval' 0 0 `ticval'
*/        

* add tic marks
        local rlst = `rhead' - 200
        local rlend = `rlst' + 150
        gph line `rlst' `cval' `rlend' `cval'
        local rlst = `rhead' + (`nplot'*`rvar') - 200
        local rlend = `rlst' - 150
        gph line `rlst' `cval' `rlend' `cval'

* lower tick numbers        
        local ticval : display round(`minb' + ((`itic'-1)*`nper'),.01)
        local itic = `itic' + 1
        local rloc = `rhead' + ((`ivar'-1)*`rvar') + 500
        gph text `rloc' `cval' 0 0 `ticval'
    }

* line at bottom of plot
    local rloc = `rhead' + ((`ivar'-1)*`rvar') - 200
    gph line `rloc' `cstrt' `rloc' `cend'

* plot letters
    gph font 700 350
    gph vtext rchange cchange cltr

* add names and types of change coefficients
    local ivar = 1
    while `ivar' <= `nplot' {
        * name of variable
        local varnum = `toplot'[`ivar',1]
        local vname: word `varnum' of `varnm'
        local rloc = `rhead' + `rnmoff' + (`ivar'*`rvar') + 0
        /* + at end moves letters down */
        gph font 600 300        
        if `istype'[`ivar',1] == 1 { /* range */
            gph text `rloc' `cname' 0 1 `vname': range
        }
        else if `istype'[`ivar',1] == 2 { /* 0 to 1 */
            gph text `rloc' `cname' 0 1 `vname': 0->1
        }
        else if `istype'[`ivar',1] == 3 { /* +-1 */
            gph text `rloc' `cname' 0 1 `vname': +/-.5
        }
        else if `istype'[`ivar',1] == 4 { /* +-sd */
            gph text `rloc' `cname' 0 1 `vname': +/-sd/2
        }
/*
* dividing line between variables
        if `ivar' != 1 {
            local rloc = `rhead' + ((`ivar'-1)*`rvar') - 200
            gph line `rloc' `cstrt' `rloc' `cend'
        }
*/
        local ivar = `ivar' + 1
        }
    gph close
end


