capture program drop oddsplot
*! version 1.0.1 jsl 7/16/97
*! version 1.0.0 jsl 10/24/96
*
* oddsplot - odds ratio plot
*
* oddsplot [packed] ([# s|u],[# s|u]...)
*
* Note: you can list the same variable more than once, for example
*
*    oddsplot 1 s 1 u
*

program define oddsplot
    version 5.0

    local input = "`*'"

* this allow running the program several times after a mlogit
    capture drop bplt bvar bcat boff bstd bltr cbeta rbeta
    tempvar b bs sd isstd toplot sdi offset
    tempvar bplt bvar bcat boff bstd bchk

* get data from logit
    if "$S_E_cmd"!="mlogit" {
        di in red "-oddsplot- must be run after -mlogit-."
        exit
    }

    mat `b' = get(_b)
    local varnm : colnames(`b')
    parse "`varnm'", parse (" ")
    quietly getdesc
    mat `sd' = S_2[1,2...]
    local nvarp1 = colsof(`b') /* with constant */
    local nvar = `nvarp1' - 1 /* excludes constant */
    local ncat = rowsof(`b') + 1
    local maxnvar = 3*`nvar'

* parse std and unstd information
    mat `isstd' = J(`maxnvar',1,0)  /* 1 if plt std coef */
    mat `toplot' = J(`maxnvar',1,0)  /* 1 if plot this coef */
    local nplot = 0
    parse "`input'", parse(" ,")

    local packit = 0
    local maxvar = 5
    local i = 1
    while "``i''"~="" {
        if "``i''"=="packed" {
            local packit = 1
            local maxvar = 11 /* max # to plot per graph */
        }
        local i = `i'+1
    }

* if not variables specifed, take first 5 and plot unstd
    if "`1'"=="" {
        local ivar = 1
        while `ivar' <= `nvar' {
            if `ivar' <= `maxvar' {
                mat `toplot'[`ivar',1] = `ivar'
                }
            local ivar = `ivar' + 1
            }
        local nplot = `ivar' - 1
        }
    else {
        while "`1'"~="" & "`1'"~="," & "`1'"~="packed" {
            if `nplot' >= `maxvar' {
                di in red "Only `maxvar' variables can be plotted at one time."
                exit
            }

            if `1' > `nvar' {
                di in red "Error: Variable number is too large."
                exit
            }
            else {
                local ntoplot = `ntoplot' + 1
                mat `toplot'[`ntoplot',1] = `1' /* plt var in this order */
            }
            macro shift
            if "`1'" == "s" {
                mat `isstd'[`ntoplot',1] = 1
            }
            macro shift
            local nplot = `nplot' + 1
        }
    }

* compute standardized coefficients
    mat `bs' = `b'
    local ivar = 1
    while `ivar' < `nvarp1' {
        * loop over categories and fill in border
        local icat = 1
        scalar `sdi' = `sd'[1,`ivar']
        while `icat' < `ncat' {
            mat `bs'[`icat',`ivar'] = `b'[`icat',`ivar'] * `sdi'
            local icat = `icat' + 1
            }
        local ivar = `ivar' + 1
        }

* stack coefficients to plot

    local nout = `nplot' * `ncat'  /* # coefs to plot */
    matrix `bplt' = J(`nout',1,0)  /* coef to plot */
    matrix `boff' = J(`nout',1,0)  /* vertical offset */
    matrix `bvar' = J(`nout',1,0)  /* variable nubmer */
    matrix `bcat' = J(`nout',1,0)  /* category */
    matrix `bstd' = J(`nout',1,0)  /* category */
    * loop through variables, decide on std and unstd to plot
    local iloc = 1
    local ivar = 1
    while `ivar' <= `nplot' {
        local icat = 1
        local varnum = `toplot'[`ivar',1]
        while `icat' <= `ncat' {
            if `icat' < `ncat' {
                matrix `bplt'[`iloc',1] = `b'[`icat',`varnum']
                matrix `bstd'[`iloc',1] = 0
                if `isstd'[`ivar',1] == 1 {
                    matrix `bplt'[`iloc',1] = `bs'[`icat',`varnum']
                    matrix `bstd'[`iloc',1] = 1
                }
            }
            else {
                matrix `bplt'[`iloc',1] = 0 /* reference category */
                matrix `bstd'[`iloc',1] = 0
                if `isstd'[`ivar',1] == 1 {
                    matrix `bstd'[`iloc',1] = 1
                }
            }
            * bplt now has coefficients to be plotted
            matrix `bvar'[`iloc',1] = `ivar' /* var# for given coef */
            matrix `bcat'[`iloc',1] = `icat' /* cat# for tiven coef */
            local iloc = `iloc' + 1
            local icat = `icat' + 1
            }
        local ivar = `ivar' + 1
        }

* determine vertical offsets for letters
*
* the algorithm is to use offsets of 0, 1, 2, 0, 1, 2 as coefficients
* are ordered from smallest to largerst.  The code is messy, but
* works...

* if packit == 1, use 0's already in boff

    if `packit'== 0 {
        local iloc = 1
        local ivar = 1
        while `ivar' <= `nplot' {
            local varnum = `toplot'[`ivar',1]
            * grab coefficients for given variable in bplt
            local istrt = ((`ivar'-1)*`ncat') + 1
            local iend = `istrt' + `ncat' - 1
            matrix `bchk' = `bplt'[`istrt'..`iend',1]
            local icat = 1
            while `icat' <= `ncat' {
                local icat2 = 1
                local minn = 99999999
                * find smallest coef that hasn't been used
                while `icat2' <= `ncat' {
                    if `bchk'[`icat2',1] < `minn' {
                        local minn = `bchk'[`icat2',1]
                        local minloc = `icat2'
                    }
                    local icat2 = `icat2' + 1
                }
                * this is current smallest
                local updown = mod(`icat'+2,3)
                * change to big values so won't select again
                mat `bchk'[`minloc',1] = 99999999
                local minn = 99999999
                * this is the location in the stored variables
                local ichng = `istrt' + `minloc' - 1
                mat `boff'[`ichng',1] = `updown'
                local iloc = `iloc' + 1
                local icat = `icat' + 1
            }
            local ivar = `ivar' + 1
        }
    }

* create variables from the matrices
    mat colnames `bplt' = bplt
    mat colnames `bvar' = bvar
    mat colnames `bcat' = bcat
    mat colnames `boff' = boff
    mat colnames `bstd' = bstd
    svmat `bplt', names(col)
    svmat `bvar', names(col)
    svmat `bcat', names(col)
    svmat `boff', names(col)
    svmat `bstd', names(col)

* generate variable with first letter of category names
    quietly catnm $S_E_depv
    local refcat "$S_2"
    local catnm  "$S_1"
    local ltr : word `refcat' of `catnm'                
    local ltr = upper(substr("`ltr'",1,1))
    quietly generate str1 bltr = "`ltr'"
    
    local icat = 1
    while `icat' <= `ncat' {
        if `icat'!=`refcat' {
            local nm: word `icat' of `catnm'
            local ltr = upper(substr("`nm'",1,1))
            quietly replace bltr = "`ltr'" if bcat == `icat'
        }
        local icat = `icat' + 1        
    }

* plot results.
*
*   (0,0)       (0,cstrt)                   (0,cend)
*
*               (rhead,cstrt)      data     (rhead,cend)
*
*               (rhead+rvar,cstrt)          (rhead+rvar,cend)
*
* parameters used for locating plot

    local cstrt  = 10000  /* first column in plot space */
    local cborder = 1000  /* border at ends of plot space */
    local cend   = 31500  /* right most column of plot space */
    local crange = `cend' - `cstrt' - (2*`cborder')
    local cname  = `cstrt' - 700  /* name of variable ends here */
    local rhead  = 2000   /* header space for factor change scale */
    local rvar   = 3600   /* rows per variable in plot space */
    if `packit' == 1 {
        local rvar   = 1700   /* rows per variable in plot space */
        }
    local rnmoff = 1000 - `rvar'  /* vert offset for listing name */
    local rltoff = 1200   /* vert offset top & bot for letters */
    local rltoff = 800   /* vert offset top & bot for letters */

    local ntic = 7        /* number tic marks */

* rescale to metric used in columns of plot
    quietly sum bplt /* get min and max of coefficients being plotted */
    local minb = _result(5)
    local maxb = _result(6)
    local rng = `maxb' - `minb'
    * change range to 0--1
    quietly generate cbeta = (bplt - `minb')/`rng'
    * min and max column for plotting area
    local minn = `cstrt' + `cborder'
    local maxx = `cend' - `cborder'
    local rng = `maxx' - `minn'
    quietly replace cbeta = (cbeta * `rng') + `minn'

* rescale metric for rows in plot
    * number of rows to contain letters
    local rltr = `rvar' - `rltoff' - `rltoff'
if `packit' == 0 {
    quietly sum boff
    local minn = _result(5)
    local maxx = _result(6)
    local rng = `maxx' - `minn'
    * change to range of 0--1
    quietly generate rbeta = (boff - `minn')/`rng'
    * expand to range of rltr
    replace rbeta = rbeta * `rltr' /* offsets as # of rows */
    * determine row position
    #delimit ;
    replace rbeta = rbeta
                  + `rhead'            /* skip over header */
                  + ((bvar-1)*`rvar')  /* space over prior variables */
                  + `rltoff'           /* space before letters begin */
                  ;
    #delimit cr
    }
else if `packit' == 1 {
    * determine row position
    #delimit ;
    generate rbeta = `rhead'            /* skip over header */
                  + ((bvar-1)*`rvar')  /* space over prior variables */
                  + `rltoff'           /* space before letters begin */
                  ;
    #delimit cr
}
* make the graph
    gph open
    gph font 600 300
* axis at top of graph
    local rloc = `rhead' - 200
    gph line `rloc' `cstrt' `rloc' `cend'
* label at top of graph
    gph font 500 250
    local rloc = `rhead' - 1400
* label at bottom of graph
    local ltr = upper("$jsl_rfnm")
    gph text `rloc' `cstrt' 0 -1 Factor Change Scale Relative to `ltr'
    local rloc = `rhead' + ((`ivar'-1)*`rvar') + 1200
    gph text `rloc' `cstrt' 0 -1 Logit Coefficient Scale Relative to `ltr'
* add tic marks
    local nper = (`maxb' - `minb')/(`ntic' - 1)
    local itic = 1
    local minn = `cstrt' + `cborder'
    local maxx = `cend' - `cborder'
    local cper = (`maxx' - `minn')/(`ntic' - 1)
    while `itic' <= `ntic' {
        local ticval : display round(exp(`minb' + ((`itic'-1)*`nper')),.01)
*        local ticval = int(`ticval'*100)/100
        local cval = `minn' + ((`itic'-1)*`cper')
        local rloc = `rhead' - 600
        gph text `rloc' `cval' 0 0 `ticval'
        * add tic marks
        local rlst = `rhead' - 200
        local rlend = `rlst' + 150
        gph line `rlst' `cval' `rlend' `cval'
        local rlst = `rhead' + (`nplot'*`rvar') - 200
        local rlend = `rlst' - 150
        gph line `rlst' `cval' `rlend' `cval'
        local ticval : display round(`minb' + ((`itic'-1)*`nper'),.01)
*        local ticval = int(`ticval'*100)/100
        local itic = `itic' + 1
        local rloc = `rhead' + ((`ivar'-1)*`rvar') + 400
        gph text `rloc' `cval' 0 0 `ticval'
        }
    gph font 700 350
    local rloc = `rhead' + ((`ivar'-1)*`rvar') - 200
    gph line `rloc' `cstrt' `rloc' `cend'

* plot letters
    gph vtext rbeta cbeta bltr

* add names and dividing lines
    local ivar = 1
    while `ivar' <= `nplot' {
        * name of variable
        local varnum = `toplot'[`ivar',1]
        local vname: word `varnum' of `varnm'
        local rloc = `rhead' + `rnmoff' + (`ivar'*`rvar')
        gph font 700 350
if `packit' == 1 {
        if `isstd'[`ivar',1] == 1 {
            gph text `rloc' `cname' 0 1 `vname'-std
            }
        else if `isstd'[`ivar',1] == 0 {
            gph text `rloc' `cname' 0 1 `vname'
            }
        }
if `packit' == 0 {
        gph text `rloc' `cname' 0 1 `vname'
        if `isstd'[`ivar',1] == 1 {
            gph font 400 200
            local rloc = `rhead' + `rnmoff' + (`ivar'*`rvar') + 900
            gph text `rloc' `cname' 0 1 Std Coef
            }
}
        * dividing line
        if `ivar' != 1 {
            local rloc = `rhead' + ((`ivar'-1)*`rvar') - 200
            gph line `rloc' `cstrt' `rloc' `cend'
            }
        local ivar = `ivar' + 1
        }

    gph close

end


