capture program drop p_val
*! 1.1.0 - use catnm to get reference values
*! version 1.0.7 4/9/97 - add nbreg
* version 1.0.6 4/6/97 - add poisson
* version 1.0.5 9/3/96
* p_val: computes predicated values
*
* Used by: change and predval programs
* Input: S_1 holds values of x
* Output:
* S_1 returns predicted value
*  : x*b for logit, probit, ologit, oprobit, tobit, regress
*  : exp(x*b) for poisson and negative binomial
* S_2 returns predicted probabilities
* Works for logit, probit, ologit, oprobit, mlogit, poisson, nbreg, regress
*
* do: column names for ologit and oprobit? values of categores?
* do: error if not following valid S_E_cmd

program define p_val
    version 4.0

    tempvar b x o xb sxb prob marg tmp bxb bxbm1 difpdf tmp2 bi alpha
    tempvar sumxb sp ai gai
    mat `b' = get(_b)
    mat `x' = S_1  /* assumes values of x in S_1 */
    mat S_1 = J(1,1,1)

* Negative Binomial
if "$S_E_cmd"=="nbreg" {
    local nb = colsof(`b')
    scalar `alpha' = exp(`b'[1,`nb'])
    scalar `ai' = 1/`alpha'
    scalar `gai' = exp(lngamma(`ai'))    
    local nb = `nb' - 1 /* -1 for alpha */        
    mat `b' = `b'[1,1..`nb']
    mat `o'= J(1,1,1)
    mat `x' = `x',`o'
    mat `xb' = `x' * `b''
    scalar `sxb' = exp(`xb'[1,1])
    mat S_1[1,1] = `sxb'
    mat colnames S_1 = exp(xb)
    mat S_2 = J(1,10,1)
    mat colnames S_2 = =0 =1 =2 =3 =4 =5 =6 =7 =8 =9
    local i = 0
    while `i' <= 9 {
        local i1 = `i' + 1
        scalar `prob' = (  exp(lngamma(`i'+`ai')) /*
    */ / ( round(exp(lnfact(`i')),1) * exp(lngamma(`ai')) )  ) /*
    */ * ((`ai'/(`ai'+`sxb'))^`ai') * ((`sxb'/(`ai'+`sxb'))^`i')
        mat S_2[1,`i1'] = `prob'
        local i=`i'+1
    }
}

* regress
if "$S_E_cmd"=="regress" {
    mat `o'= J(1,1,1)
    mat `x' = `x',`o'
    mat `xb' = `x' * `b''
    scalar `sxb' = `xb'[1,1]
    mat S_1[1,1] = `sxb'
    mat colnames S_1 = xb    
    mat S_2 = J(1,1,-99999)
    mat colnames S_1 = void
}

* poisson
if "$S_E_cmd"=="poisson" {
    mat `o'= J(1,1,1)
    mat `x' = `x',`o'
    mat `xb' = `x' * `b''
    scalar `sxb' = exp(`xb'[1,1])
    mat S_1[1,1] = `sxb'
    mat colnames S_1 = exp(xb)
    mat S_2 = J(1,10,1)
    mat colnames S_2 = =0 =1 =2 =3 =4 =5 =6 =7 =8 =9
    local i = 0
    while `i' <= 9 {
        local i1 = `i' + 1
        scalar `prob' = exp(-`sxb')*(`sxb'^`i')/round(exp(lnfact(`i')),1)
        mat S_2[1,`i1'] = `prob'
        local i=`i'+1
    }
}

* binary logit and probit
if "$S_E_cmd"=="logit" | "$S_E_cmd"=="probit" {
    mat `o'= J(1,1,1)
    mat `x' = `x',`o'
    mat `xb' = `x' * `b''
    local nb = colsof(`b') - 1
    mat `b' = `b'[1,1..`nb'] /* get rid of _con */
    scalar `sxb' = `xb'[1,1]
    mat S_1[1,1] = `sxb'
    mat colnames S_1 = xb
    if "$S_E_cmd"=="logit" {
         scalar `prob' = exp(`sxb')/(1+exp(`sxb'))
    }
    else if "$S_E_cmd"=="probit" {
        scalar `prob' = normprob(`sxb')
    }
    mat S_2 = J(1,2,1)
    mat colnames S_2 = =0 =1
    mat S_2[1,2] = `prob'        /* prob of 1 */
    mat S_2[1,1] = 1-`prob'      /* prob of 0 */
}

* ordered probit and ordered logit
if "$S_E_cmd"=="oprobit" | "$S_E_cmd"=="ologit" {
* determine number of categories in lhs variable
    local ncats 1
    capture local j=_b[_cut`ncats']
    while _rc==0 {
        local ncats=`ncats'+1
        capture local j=_b[_cut`ncats']
    }
* compute xb without intercept
    mat `b' = get(_b)
    local nb = colsof(`b') - `ncats'
    local nb = `nb' + 1 /* # of b's excluding constants */
    mat `b' = `b'[1,1..`nb'] /* get rid of _con's */
    mat `xb' = `x' * `b''
    scalar `sxb' = `xb'[1,1]
    mat S_1[1,1] = `sxb'
    mat colnames S_1 = xb    
* compute probabilities
    if "$S_E_cmd"=="oprobit" {
        mat `prob' = J(1,`ncats',1)
        scalar `bxb' = _b[_cut1]-`sxb'
        scalar `tmp' = normprob(`bxb') /* prob for cat 1 */
        mat `prob'[1,1] = `tmp'
        local i 2
        while `i'<`ncats' {
            local im1=`i'-1
            scalar `bxb' = _b[_cut`i']-`sxb'
            scalar `bxbm1' = _b[_cut`im1']-`sxb'
            scalar `tmp' = normprob(`bxb') - normprob(`bxbm1')
            mat `prob'[1,`i'] = `tmp'
            local i=`i'+1
        }
        local im1=`i'-1
        scalar `bxb' = `sxb'-_b[_cut`im1']
        scalar `tmp' = normprob(`bxb')
        mat `prob'[1,`ncats'] = `tmp'
    }
    if "$S_E_cmd"=="ologit" {
        mat `prob' = J(1,`ncats',1)
        scalar `tmp' = 1/(1+exp(`sxb'-_b[_cut1]))
        mat `prob'[1,1] = `tmp'
        local i 2
        while `i'<`ncats' {
            local im1=`i'-1
            scalar `tmp' = 1/(1+exp(`sxb'-_b[_cut`i']))
            scalar `tmp2' = 1/(1+exp(`sxb'-_b[_cut`im1']))
            mat `prob'[1,`i'] = `tmp' - `tmp2'
            local i=`i'+1
        }
        local im1=`i'-1
        scalar `tmp' = 1/(1+exp(`sxb'-_b[_cut`im1']))
        mat `prob'[1,`ncats'] = 1-`tmp'
    }
    mat S_2 = `prob'
}

*
* mlogit
* NOTE:  Assumes setcat has been run with these in memory:
* global macros:
* jsl_rf == ref category
* jsl_rfnm == name of reference category
* jsl_ctnm: names of cateories
* jsl_ct: cats in order in analysis, excluding refcat

if "$S_E_cmd"=="mlogit" {
* determine number of categories in lhs variable
    mat `b' = get(_b)
    local nb = colsof(`b')
    local ncats = rowsof(`b') + 1
* get xb over all sets of b's
    mat `o'= J(1,1,1)
    mat `x' = `x',`o'
    mat `prob' = J(1,`ncats',-1)
    mat `xb' = `b'*`x'' /* one row for each set of b's */
    mat S_1 = `xb'
mat colnames S_1 = xb    
    scalar `sumxb' = 1
    local i = 1
    while `i' < `ncats' {
        scalar `sxb' = exp(`xb'[`i',1])
        scalar `sumxb' = `sumxb' + `sxb' /* sum of exp(xb) */
        mat `prob'[1,`i'] = `sxb'
        local i = `i' + 1
    }
* put prob of ref category in last location
    scalar `sumxb' = 1/`sumxb'
    mat `prob'[1,`ncats'] = 1
    mat `prob' = `sumxb'*`prob'
    mat S_2 = `prob'
*    mat colnames S_2 = $jsl_ctnm $jsl_rfnm
}
    
end

/*
mlogit occ white ed exp,basecategory(5)
setcat 5 prof 1 menial 2 blueC 3 craft 4 whiteC
mat S_1 = (1,11,3)
mat list S_1
di ".168 .412 .276 .085 .0576"
p_val
mat list S_1
*/

